/* a few routines from mt-process.c needed by trn now */
#include <stdio.h>
#include <string.h>

#define NAMELEN 16
#define SUBJLEN 72

char	subject_str[80];
int	found_Re;

char	author_str[20];

#define EQ(x,y) ((isupper(x) ? tolower(x) : (x)) == (y))

/*
 * Parse the subject into 72 characters or less.  Remove any "Re[:^]"s from
 * the front (noting that it's there), and any "(was: old)" stuff from the
 * end.  Then, compact multiple whitespace characters into one space,
 * trimming leading/trailing whitespace.  If it's still too long,
 * unmercifully cut it off.  We don't bother with subject continuation
 * lines either.
 */
void
get_subject_str(str)
register char *str;
{
	register char *cp;
	register int len;

	while (*str && (unsigned char)*str <= ' ')
		str++;
	if (*str == '\0') {
		(void) strcpy(subject_str, "<None>");
		return;					/* RETURN */
	}
	cp = str;
	while (EQ(cp[0], 'r') && EQ(cp[1], 'e')) {	/* check for Re: */
		cp += 2;
		if (*cp == '^')				/* allow Re^2: */
			while (*++cp <= '9' && *cp >= '0')
				;
		if (*cp != ':')
			break;
		while (*++cp == ' ')
			;
		found_Re = 1;
		str = cp;
	}
	/*
	 * Remove "(was: oldsubject)", because we already know the old
	 * subjects.  Also match "(Re: oldsubject)".  Allow possible spaces
	 * after the ('s.
	 */
	for (cp = str; (cp = strchr(cp + 1, '(')) != NULL; ) {
		while (*++cp == ' ')
			;
		if (EQ(cp[0], 'w') && EQ(cp[1], 'a') && EQ(cp[2], 's')
		    && (cp[3] == ':' || cp[3] == ' ')) {
			*--cp = '\0';
			break;
		}
		if (EQ(cp[0], 'r') && EQ(cp[1], 'e') &&
		    (cp[2] == ':' && cp[3] == ' ' ||
		     cp[2] == '^' && cp[4] == ':')) {
			*--cp = '\0';
			break;
		}
	}
	/* Copy subject to a temporary string, compacting multiple spaces/tabs */
	for (len = 0, cp = subject_str; len < SUBJLEN && *str; len++)
		if ((unsigned char)*str <= ' ') {
			while (*++str && (unsigned char)*str <= ' ')
				;
			*cp++ = ' ';
		} else
			*cp++ = *str++;
	if (cp[-1] == ' ')
		cp--;
	*cp = '\0';
}

/*
 * Try to fit the author name in 16 bytes.  Use the comment portion in
 * parenthesis if present.  Cut off non-commented names at the '@' or '%'.
 * Then, put as many characters as we can into the 16 bytes, packing
 * multiple whitespace characters into a single space.  We might want to
 * implement a nice name shortening algorithm sometime.
 */
void
get_author_str(str)
char *str;
{
	register char *cp, *cp2;

	if ((cp = strchr(str, '(')) != NULL) {
		str = cp + 1;
		if ((cp = strrchr(str, ')')) != NULL)
			*cp = '\0';
	} else {
		if ((cp = strchr(str, '@')) != NULL)
			*cp = '\0';
		if ((cp = strchr(str, '%')) != NULL)
			*cp = '\0';
	}
	for (cp = str, cp2 = author_str; *cp && cp2 - author_str < NAMELEN; )
		/* Pack white space and turn ctrl-chars into spaces. */
		if (*cp <= ' ') {
			while (*++cp && *cp <= ' ')
				;
			if (cp2 != author_str)
				*cp2++ = ' ';
		} else
			*cp2++ = *cp++;
	*cp2 = '\0';
}
